<?php
/**
 * Local Compress
 * @since 5.00.59
 */


class wps_local_compress
{

  private static $allowed_types;
  private static $apiURL;
  private static $siteUrl;
  private static $apiParams;
  private static $settings;
  private static $options;
  private static $zone_name;
  private static $backup_directory;
  public $webp_sizes;
  public $sizes;
  public $total_sizes;
  public $compressed_list;

  public $enabledLog;
  public $logFile;


  public function __construct()
  {
    global $wps_ic;
    global $wpc_filesystem;

    $this->enabledLog = 'true';

    $this->logFilePath = WPS_IC_DIR . 'compress-log.txt';
    $this->logFile = fopen($this->logFilePath, 'a');

    $this->get_filesystem();

    $this->total_sizes = count(get_intermediate_image_sizes());
    $this->sizes = $this->getAllThumbSizes();
    $this->webp_sizes = get_intermediate_image_sizes();
    $uploads_dir = wp_upload_dir();

    self::$allowed_types = array('jpg' => 'jpg', 'jpeg' => 'jpeg', 'gif' => 'gif', 'png' => 'png');
    self::$backup_directory = $uploads_dir['basedir'] . '/wp-compress-backups';
    self::$settings = get_option(WPS_IC_SETTINGS);
    self::$options = get_option(WPS_IC_OPTIONS);
    self::$siteUrl = site_url();

    /**
     * If backup directories don't exist, create them
     */
    if (!file_exists(self::$backup_directory)) {
      $made_dir = mkdir(self::$backup_directory, 0755);
      if (!$made_dir) {
        update_option('wpc_errors', array('unable-to-create-backup-dir' => self::$backup_directory));
      } else {
        delete_option('wpc_errors');
      }
    }

    add_action('delete_attachment', array($this, 'on_delete'));

    if (!empty(self::$settings['on-upload']) && self::$settings['on-upload'] == '1' && empty($_GET['restoreImage'])) {
      /*
       * This works but uploads a full sized image to storage for every size variation
       */

      add_filter('wp_generate_attachment_metadata', array($this, 'on_upload'), PHP_INT_MAX, 2);
      // TODO: Causing problems with showing 0% saved, while actually compressed
    }

    if (empty(self::$settings['cname']) || !self::$settings['cname']) {
      self::$zone_name = get_option('ic_cdn_zone_name');
    } else {
      self::$zone_name = get_option('ic_custom_cname');
    }

    $location = get_option('wps_ic_geo_locate');
    if (empty($location)) {
      $location = $this->geoLocate();
    }

    if (is_object($location)) {
      $location = (array)$location;
    }

    $apiVersion = 'v4';

    if (!empty($_GET['force_region'])) {
      $location = get_option('wps_ic_geo_locate');
      $location->continent = $_GET['force_region'];
      $location->custom_server = $_GET['custom_server'];
      $location = (array)$location;
      update_option('wps_ic_geo_locate', $location);
    }

    if (!empty($_GET['reset_region'])) {
      $call = wp_remote_get('https://cdn.zapwp.net/?action=geo_locate&domain=' . urlencode(site_url()), ['timeout' => 30, 'sslverify' => false, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0']);

      if (wp_remote_retrieve_response_code($call) == 200) {
        $body = wp_remote_retrieve_body($call);
        $body = json_decode($body);

        if ($body->success) {
          update_option('wps_ic_geo_locate', $body->data);
        } else {
          update_option('wps_ic_geo_locate', ['country' => 'EU', 'server' => 'frankfurt.zapwp.net']);
        }
      } else {
        update_option('wps_ic_geo_locate', ['country' => 'EU', 'server' => 'frankfurt.zapwp.net']);
      }
    }

    if (isset($location) && !empty($location)) {
      if (is_array($location) && !empty($location['server'])) {

        if (empty($location['continent'])) {
          $location['continent'] = '';
        }

        if ($location['continent'] == 'CUSTOM') {
          self::$apiURL = 'https://' . $location['custom_server'] . '.zapwp.net/local/' . $apiVersion . '/';
        } elseif ($location['continent'] == 'AS' || $location['continent'] == 'IN') {
          self::$apiURL = 'https://singapore.zapwp.net/local/' . $apiVersion . '/';
        } elseif ($location['continent'] == 'EU') {
          self::$apiURL = 'https://germany.zapwp.net/local/' . $apiVersion . '/';
        } elseif ($location['continent'] == 'OC') {
          self::$apiURL = 'https://sydney.zapwp.net/local/' . $apiVersion . '/';
        } elseif ($location['continent'] == 'US' || $location['continent'] == 'NA' || $location['continent'] == 'SA') {
          self::$apiURL = 'https://nyc.zapwp.net/local/' . $apiVersion . '/';
        } else {
          self::$apiURL = 'https://germany.zapwp.net/local/' . $apiVersion . '/';
        }
      } else {
        self::$apiURL = 'https://' . $location->server . '/local/' . $apiVersion . '/';
      }
    } else {
      self::$apiURL = 'https://germany.zapwp.net/local/' . $apiVersion . '/';
    }

	  $local_server = get_option('wps_ic_force_local_server');
	  if ($local_server !== false && $local_server !== 'auto'){
		  self::$apiURL = 'https://' . $local_server . '/local/' . $apiVersion . '/';
	  }

    // Setup paraams for POST to API
    self::$apiParams = array();
    self::$apiParams['apikey'] = self::$options['api_key'];
    self::$apiParams['quality'] = self::$settings['optimization'];
    self::$apiParams['retina'] = 'false';
    self::$apiParams['webp'] = 'false';
    self::$apiParams['width'] = 'false';
    self::$apiParams['url'] = '';
  }

  public function get_filesystem()
  {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-filesystem-base.php');
    require_once(ABSPATH . 'wp-admin/includes/class-wp-filesystem-direct.php');
    global $wpc_filesystem;

    if (!defined('FS_CHMOD_DIR')) {
      define('FS_CHMOD_DIR', (fileperms(ABSPATH) & 0777 | 0755));
    }

    if (!defined('FS_CHMOD_FILE')) {
      define('FS_CHMOD_FILE', (fileperms(ABSPATH . 'index.php') & 0777 | 0644));
    }

    if (!isset($wpc_filesystem) || !is_object($wpc_filesystem)) {
      $wpc_filesystem = new WP_Filesystem_Direct('');
    }
  }

  public function getAllThumbSizes()
  {
    global $_wp_additional_image_sizes;

    $default_image_sizes = get_intermediate_image_sizes();

    foreach ($default_image_sizes as $size) {
      $image_sizes[$size]['width'] = intval(get_option("{$size}_size_w"));
      $image_sizes[$size]['height'] = intval(get_option("{$size}_size_h"));
      $image_sizes[$size]['crop'] = get_option("{$size}_crop") ? get_option("{$size}_crop") : false;
    }

    if (isset($_wp_additional_image_sizes) && count($_wp_additional_image_sizes)) {
      $image_sizes = array_merge($image_sizes, $_wp_additional_image_sizes);
    }

    $AdditionalSizes = array('full');
    foreach ($AdditionalSizes as $size) {
      $image_sizes[$size]['width'] = 'full';
    }

    $image_sizes['original']['width'] = 'original';

    return $image_sizes;
  }

  public function geoLocate()
  {
    $force_location = get_option('wpc-ic-force-location');
    if (!empty($force_location)) {
      return $force_location;
    }

    $call = wp_remote_get('https://cdn.zapwp.net/?action=geo_locate&domain=' . urlencode(site_url()), array('timeout' => 30, 'sslverify' => false, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));
    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);

      if ($body->success) {
        update_option('wps_ic_geo_locate', $body->data);

        return $body->data;
      } else {
        update_option('wps_ic_geo_locate', array('country' => 'EU', 'server' => 'frankfurt.zapwp.net'));

        return array('country' => 'EU', 'server' => 'frankfurt.zapwp.net');
      }
    } else {
      update_option('wps_ic_geo_locate', array('country' => 'EU', 'server' => 'frankfurt.zapwp.net'));

      return array('country' => 'EU', 'server' => 'frankfurt.zapwp.net');
    }
  }

  public function on_delete($post_id)
  {
	  // Delete webP if exists
	  $imagesCompressed = get_post_meta($post_id, 'wpc_images_compressed', true);
    if (!empty($imagesCompressed) && is_array($imagesCompressed)) {
      foreach ($imagesCompressed as $image => $data) {
        if (file_exists($data['webp_path'])) {
          unlink($data['webp_path']);
        }
      }
    }
  }

  public function on_upload_gutenber($meta, $id)
  {
    global $wpc_filesystem;

    if (!$this->is_supported($id)) {
      return $meta;
    }

    $attachment_id = $id;
    wp_raise_memory_limit('image');

    $this->backup_image($attachment_id);

    $image = wp_get_attachment_image_src($attachment_id, 'full');
    $file_path = get_attached_file($attachment_id);
    $file_basename = basename($image[0]);

    // Figure out image type
    $exif = exif_imagetype($file_path);
    $mime = image_type_to_mime_type($exif);

    // Fetch the image content
    $file_content = $wpc_filesystem->get_contents($file_path);

    $post_fields = array('action' => 'compress', 'imageID' => $attachment_id, 'filename' => $file_basename, 'apikey' => self::$apiParams['apikey'], 'key' => self::$apiParams['apikey'], 'image' => $image[0], 'url' => $image[0], 'exif' => $exif, 'mime' => $mime, 'content' => base64_encode($file_content), 'quality' => self::$apiParams['quality'], 'retina' => self::$apiParams['retina'], 'webp' => self::$apiParams['webp'], 'count_thumbs' => $this->total_sizes);

    $tmp_location = $file_path . '_tmp';
    $file_location = $file_path;
    $original_filesize = filesize($file_path);
    $response = wp_remote_post(self::$apiURL, array('timeout' => 300, 'method' => 'POST', 'sslverify' => false, 'body' => $post_fields, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));

    if (wp_remote_retrieve_response_code($response) == 200) {
      $body = wp_remote_retrieve_body($response);
      clearstatcache();

      if (!empty($body)) {
        file_put_contents($tmp_location, $body);
        unset($body);

        // Check if compressed image is smaller than original image from backup
        $compressed_filesize = filesize($tmp_location);

        if ($compressed_filesize >= $original_filesize) {
          // Do Nothing
        } else {
          unlink($file_location);
          copy($tmp_location, $file_location);
          unlink($tmp_location);

          $stats['full']['original']['size'] = $original_filesize;
          $stats['full']['compressed']['size'] = $compressed_filesize;

          update_post_meta($attachment_id, 'ic_status', 'compressed');
          update_post_meta($attachment_id, 'ic_stats', $stats);
        }
      }
    }

    update_post_meta($attachment_id, 'is_uploaded', 'true');

    return $meta;
  }

  public function is_supported($imageID)
  {
    $file_data = get_attached_file($imageID);
    $type = wp_check_filetype($file_data);

    // Is file extension allowed
    if (!in_array(strtolower($type['ext']), self::$allowed_types)) {
      return false;
    } else {
      return true;
    }
  }

  public function backup_image($imageID)
  {
    wp_raise_memory_limit('image');

    if (empty(self::$settings['backup']['local'] || self::$settings['backup']['local'] === '0')) {
      return true;
    }

    // Image Backup Exists
    if ($this->backup_exists($imageID)) {
      return true;
    }

    // Setup Image Stats
    $stats = array();
    $backup_list = array();

    // Create backup directory
    $this->create_backup_directory();

    // Get filename
    $image = wp_get_original_image_url($imageID);
    $image_url = $image;
    $parsed_url = parse_url($image_url);
    $parsed_url['path'] = ltrim($parsed_url['path'], '/');
    $filename = basename($parsed_url['path']);
    $backup_folders = str_replace($filename, '', $parsed_url['path']);
    $backup_folders = rtrim($backup_folders, '/');
    $backup_folders = explode('/', $backup_folders);

    $backup_dir = self::$backup_directory;
    if (is_array($backup_folders)) {
      foreach ($backup_folders as $i => $folder) {
        $backup_dir .= '/' . $folder;
        if (!file_exists($backup_dir)) {
          $made_dir = mkdir($backup_dir, 0755);
        }
      }
    }

    if (empty($image) || empty($image_url)) {
      return false;
    }

    // Define original / backup file paths
    $original_file_location = ABSPATH . $parsed_url['path'];

    // Where is backup saved?
    $backup_file_location = $backup_dir . '/' . $filename;

    // Stats
    $stats['original']['original']['size'] = filesize($original_file_location);

    copy($original_file_location, $backup_file_location);

    $backup_list['original'] = $backup_file_location;

    if (!file_exists($backup_file_location)) {
      // TODO: What then
      //wp_send_json_error('failed-to-create-backup');
    }

    update_post_meta($imageID, 'ic_stats', $stats);
    update_post_meta($imageID, 'ic_backup_images', $backup_list);
    update_post_meta($imageID, 'ic_original_stats', $stats);
  }


  public function backup_exists($imageID)
  {
    $backup_exists = get_post_meta($imageID, 'ic_backup_images', true);
    if (!empty($backup_exists) && is_array($backup_exists)) {
      foreach ($backup_exists as $filename => $backup_location) {
        if (!empty($backup_location)) {
          // If backup file exists
          if (file_exists($backup_location)) {
            return $backup_location;
          } else {
            return false;
          }
        }
      }

      return true;
    } else {
      return false;
    }
  }


  public function create_backup_directory()
  {
    if (!file_exists(self::$backup_directory)) {
      mkdir(self::$backup_directory, 0755);
    }
  }

  public function on_upload($data, $attachment_id)
  {
    $imageID = $attachment_id;

    // Is the image type supported
    if (!$this->is_supported($imageID)) {
      $this->writeLog('Image not supported ' . $imageID);
      return $data;
    }

    // Is the image already Compressed
    if ($this->is_already_compressed($imageID)) {
      $this->writeLog('Image not supported ' . $imageID);
      return $data;
    }

    update_post_meta($imageID, 'wpc_old_meta', $data);

    set_transient('wps_ic_compress_' . $attachment_id, ['imageID' => $attachment_id, 'status' => 'compressing'], 30);
    set_transient('wps_ic_queue_' . $attachment_id, ['imageID' => $attachment_id, 'status' => 'waiting'], 30);

	  $this->singleCompressV4($imageID, false);

    return $data;
  }

  public function writeLog($message)
  {
    if ($this->enabledLog == 'true') {
      fwrite($this->logFile, "[" . date('d.m.Y H:i:s') . "] " . $message . "\r\n");
    }
  }

  public function is_already_compressed($imageID)
  {
    $backup_exists = get_post_meta($imageID, 'ic_status', true);
    if (!empty($backup_exists) && $backup_exists == 'compressed') {
      return true;
    } else {
      return false;
    }
  }

  public function generate_webp($arg, $type = 'click')
  {
    global $wpc_filesystem;

    $upload_dir = wp_upload_dir();
    $imageID = $arg;
    $return = array();
    $compressed = array();
    $extension = '';
    $stats = array();

    $image_url_full = wp_get_attachment_image_src($imageID, 'full');
    $image_url_full = $image_url_full[0];
    $image_filename = basename($image_url_full);

    if (strpos($image_filename, '.jpg') !== false) {
      $extension = 'jpg';
    } elseif (strpos($image_filename, '.jpeg') !== false) {
      $extension = 'jpeg';
    } elseif (strpos($image_filename, '.gif') !== false) {
      $extension = 'gif';
    } elseif (strpos($image_filename, '.png') !== false) {
      $extension = 'png';
    }

    foreach ($this->webp_sizes as $i => $size) {
      if ($size == 'full') {
        $image = wp_get_attachment_image_src($imageID, $size);
        if ($image) {
          $image_url = $image[0];
        }
      } else {
        $image = wp_get_attachment_image_src($imageID, $size);
        if ($image) {
          $image_url = $image[0];
        }
      }

      if (empty($image_url)) {
        continue;
      }

      if (!isset($image['path']) && !empty($image)) {
        $image['path'] = $image;
      }

      $image['path'] = str_replace($upload_dir['baseurl'] . '/', '', $image[0]);
      $image['path'] = str_replace('./', '', $image['path']);

      /**
       * Figure out the actual file path
       */
      $file_path = get_attached_file($imageID);
      $file_basename = basename($image[0]);

      // Setup POST Params
      $headers = array('timeout' => 300, 'httpversion' => '1.0', 'blocking' => true,);

      // Figure out image type
      $exif = exif_imagetype($file_path);
      $mime = image_type_to_mime_type($exif);

      $file_location = WPS_IC_UPLOADS_DIR . '/' . $image['path'];

      // Fetch the image content
      $file_content = $wpc_filesystem->get_contents($file_path);

      $post_fields = array('action' => 'compress', 'imageID' => $imageID, 'filename' => $file_basename, 'apikey' => self::$apiParams['apikey'], 'key' => self::$apiParams['apikey'], 'image' => $image[0], 'url' => $image[0], 'exif' => $exif, 'mime' => $mime, 'content' => base64_encode($file_content), 'quality' => self::$apiParams['quality'], 'width' => '1', 'retina' => 'false', 'webp' => 'true');

      if (!empty($size)) {
        if ($size == 'full') {
          $post_fields['width'] = '1';
        } else {
          if (empty($image['width'])) {
            $post_fields['width'] = '1';
          } else {
            $post_fields['width'] = $image['width'];
          }
        }
      }

      // WebP File Path
      $webp_file_location = str_replace('.' . $extension, '.webp', $file_location);
      $call = wp_remote_post(self::$apiURL, array('timeout' => 300, 'method' => 'POST', 'headers' => $headers, 'sslverify' => false, 'body' => $post_fields, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));

      if (wp_remote_retrieve_response_code($call) == 200) {
        $body = wp_remote_retrieve_body($call);
        if (!empty($body)) {
          file_put_contents($webp_file_location, $body);
          clearstatcache();

          $stats[$size . '-webp']['compressed']['size'] = filesize($webp_file_location);
          $compressed[$size . '-webp'] = $webp_file_location;
        }
      }
    }

    $return['stats'] = $stats;
    $return['compressed'] = $compressed;

    $stats = get_post_meta($imageID, 'ic_stats', true);
    $stats = array_merge($stats, $return['stats']);
    update_post_meta($imageID, 'ic_stats', $stats);

    if ($type == 'click') {
      $compressed = get_post_meta($imageID, 'ic_compressed_images', true);
      $compressed = array_merge($compressed, $return['compressed']);
      update_post_meta($imageID, 'ic_compressed_images', $compressed);
    }

    return $return;
  }

	public function restoreV4($imageID) {

		if ( ! function_exists( 'download_url' ) ) {
			require_once( ABSPATH . "wp-admin" . '/includes/image.php' );
			require_once( ABSPATH . "wp-admin" . '/includes/file.php' );
			require_once( ABSPATH . "wp-admin" . '/includes/media.php' );
		}

		if ( ! function_exists( 'update_option' ) ) {
			require_once( ABSPATH . "wp-includes" . '/option.php' );
		}

		$output = array();

		wp_raise_memory_limit( 'image' );
		ini_set( 'memory_limit', '1024M' );

		$olderVersionBackup = $this->olderBackup( $imageID );
		if ( $olderVersionBackup ) {
			return true;
		}

		// Is the image in process
		$inProcess = get_post_meta( $imageID, 'ic_bulk_running', true );
		// Remote backup?

		//check api for original
		$params = array( 'timeout'    => 300,
		                 'method'     => 'POST',
		                 'sslverify'  => false,
		                 'body'       => [
			                 'action'   => 'restoreSingleImage',
			                 'apikey'      => self::$apiParams['apikey'],
			                 'imageID'     => $imageID
		                 ],
		                 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'
		);

		$call = wp_remote_post( self::$apiURL, $params );
		//file_put_contents( WPS_IC_CACHE . 'test3.txt', print_r( $call, true ) );
		$this->writeLog( 'Started Image ID ' . $imageID );

		if ( wp_remote_retrieve_response_code( $call ) == 200 ) {
			$response = wp_remote_retrieve_body( $call );
			$response = json_decode( $response, true );

			$this->writeLog( 'API Response IS 200' );
			$this->writeLog( print_r( wp_remote_retrieve_body( $call ), true ) );

			if ( $response['success'] == 'true' ) {
				if ( ! empty( $response['data'] ) ) {

					$alreadyRestored = [];
					$oldMeta         = get_post_meta( $imageID, 'wpc_old_meta', true );

					if ( ! empty( $response['data']['imageURL'] ) ) {
						$imageUrl  = $response['data']['imageURL'];
						$imagePath = wp_get_original_image_path( $imageID );

						$downloadImage = download_url( $imageUrl );

						if ( is_wp_error( $downloadImage ) ) {
							$this->writeLog( 'Unable to download Image' );
							$this->writeLog( $imageUrl );
							$this->writeLog( $downloadImage );

							$this->writeLog( 'Ended Image ID - failed to get backup ' . $imageID );

							if ( $output == 'json' ) {
								wp_send_json_error( array(
									'msg'     => 'failed-to-get-backup',
									'apiUrl'  => self::$apiURL,
									'apikey'  => self::$apiParams['apikey'],
									'imageID' => $imageID,
									'url'     => $downloadImage
								) );
							}

							return false;
						}

						$file_info = finfo_open(FILEINFO_MIME_TYPE);
						$mime_type = finfo_file($file_info, $downloadImage);
						finfo_close($file_info);

						if (in_array($mime_type, ['image/jpeg', 'image/png', 'image/gif'])) {
							$imageSize = getimagesize( $downloadImage );
							if ( $imageSize !== false ) {

								if ( file_exists( $imagePath ) ) {
									unlink( $imagePath );
								}

								copy( $downloadImage, $imagePath );
								unset( $downloadImage );

								// Delete webP if exists
								$imagesCompressed = get_post_meta( $imageID, 'wpc_images_compressed', true );
								foreach ( $imagesCompressed as $image => $data ) {
									if ( file_exists( $data['webp_path'] ) ) {
										unlink( $data['webp_path'] );
									}
								}


								// Remove meta tags
								delete_post_meta( $imageID, 'wpc_images_compressed' );
								delete_post_meta( $imageID, 'ic_stats' );
								delete_post_meta( $imageID, 'ic_compressed_images' );
								delete_post_meta( $imageID, 'ic_compressed_thumbs' );
								delete_post_meta( $imageID, 'ic_backup_images' );
								update_post_meta( $imageID, 'ic_status', 'restored' );
								delete_post_meta( $imageID, 'ic_bulk_running' );
								delete_transient( 'wps_ic_compress_' . $imageID );

								$originalFilePath = wp_get_original_image_path( $imageID );
								remove_filter( 'wp_generate_attachment_metadata', array( $this, 'on_upload' ), PHP_INT_MAX );
								$oldMeta = wp_generate_attachment_metadata( $imageID, $originalFilePath );
								wp_update_attachment_metadata( $imageID, $oldMeta );
								// Add for heartbeat to pickup
								set_transient( 'wps_ic_heartbeat_' . $imageID, array(
									'imageID' => $imageID,
									'status'  => 'restored'
								), 60 );

								$this->writeLog( 'Ended Image ID - restored ' . $imageID );

								if ( $output == 'json' ) {
									wp_send_json_success( array( 'msg' => 'backup-restored' ) );
								}
							}
							wp_send_json_error( array( 'msg' => 'invalid-backup' ) );
						}
						wp_send_json_error( array( 'msg' => 'invalid-backup' ) );
					}
				}
			}
		}  else {
			$this->writeLog( 'API Response not 200' );
			$this->writeLog( print_r( wp_remote_retrieve_body( $call ), true ) );
			$this->writeLog( 'Ended Image ID ' . $imageID );

			// Failure to contact API
			if ( $output == 'json' ) {
				wp_send_json_error( array( 'msg' => 'unable-to-contact-api' ) );
			}
		}
	}

  public function restore($imageID, $output = 'json')
  {
    if (!function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    if (!function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    $output = array();

    wp_raise_memory_limit('image');
    ini_set('memory_limit', '1024M');

    $olderVersionBackup = $this->olderBackup($imageID);
    if ($olderVersionBackup) {
      return true;
    }

    // Is the image in process
    $inProcess = get_post_meta($imageID, 'ic_bulk_running', true);
    if ($inProcess && $inProcess == 'true') {
    }

    // Remote backup?

    //check api for original
    $params = array('timeout' => 300, 'method' => 'POST', 'sslverify' => false, 'body' => ['getS3Backup' => true, 'apikey' => self::$apiParams['apikey'], 'imageID' => $imageID], 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0');

    $call = wp_remote_post(self::$apiURL, $params);

    $this->writeLog('Started Image ID ' . $imageID);

    if (wp_remote_retrieve_response_code($call) == 200) {
      $response = wp_remote_retrieve_body($call);
      $response = json_decode($response, true);


      $this->writeLog('API Response IS 200');
      $this->writeLog(print_r(wp_remote_retrieve_body($call), true));

      if ($response['success'] == 'true') {
        if (!empty($response['data'])) {

          $alreadyRestored = [];
          $oldMeta = get_post_meta($imageID, 'wpc_old_meta', true);

          if (!empty($response['data']['url']['original']['local'])) {
            $imageUrl = $response['data']['url']['original']['local'];
            $imagePath = wp_get_original_image_path($imageID);

            $downloadImage = download_url($imageUrl);

            if (is_wp_error($downloadImage)) {
              $this->writeLog('Unable to download Image');
              $this->writeLog($imageUrl);
              $this->writeLog($downloadImage);

              $this->writeLog('Ended Image ID - failed to get backup ' . $imageID);

              if ($output == 'json') {
                wp_send_json_error(array('msg' => 'failed-to-get-backup', 'apiUrl' => self::$apiURL, 'apikey' => self::$apiParams['apikey'], 'imageID' => $imageID, 'url' => $downloadImage));
              }

              return false;
            }

            if (file_exists($imagePath)) {
              unlink($imagePath);
            }

            copy($downloadImage, $imagePath);
            unset($downloadImage);


            // Remove meta tags
            delete_post_meta($imageID, 'wpc_images_compressed');
            delete_post_meta($imageID, 'ic_stats');
            delete_post_meta($imageID, 'ic_compressed_images');
            delete_post_meta($imageID, 'ic_compressed_thumbs');
            delete_post_meta($imageID, 'ic_backup_images');
            update_post_meta($imageID, 'ic_status', 'restored');
            delete_post_meta($imageID, 'ic_bulk_running');
            delete_transient('wps_ic_compress_' . $imageID);

	          $originalFilePath = wp_get_original_image_path($imageID);
	          remove_filter('wp_generate_attachment_metadata', array($this, 'on_upload'), PHP_INT_MAX);
	          $oldMeta = wp_generate_attachment_metadata($imageID, $originalFilePath);
	          wp_update_attachment_metadata($imageID, $oldMeta);

            // Add for heartbeat to pickup
            set_transient('wps_ic_heartbeat_' . $imageID, array('imageID' => $imageID, 'status' => 'restored'), 60);

            $this->writeLog('Ended Image ID - restored ' . $imageID);

            if ($output == 'json') {
              wp_send_json_success(array('msg' => 'backup-restored'));
            }

            return true;
          }

          foreach ($response['data']['url'] as $imageSize => $imageUrl) {

            $imageUrl = $imageUrl['s3'];

            // Image URL was already restored
            if (in_array($imageUrl, $alreadyRestored)) {
              $this->writeLog('Image was already restored');
              $this->writeLog($imageUrl);
              continue;
            }

            if ($imageSize == 'original') {
              $imagePath = wp_get_original_image_path($imageID);
            } else {
              $originalFilePath = wp_get_original_image_path($imageID);
              $originalFilename = wp_basename($originalFilePath);
              $this->pathToDir = str_replace($originalFilename, '', $originalFilePath);
              //
              $imagePath = wp_get_attachment_image_src($imageID, $imageSize);
              $imagePath = wp_basename($imagePath[0]);
              $imagePath = $this->pathToDir . $imagePath;
            }

            // Local Filename
            $localFilename = wp_basename($imagePath);

            // Filename from API
            $sentFilename = wp_basename($imageUrl);
            $sentFilename = explode('?', $sentFilename);
            $sentFilename = $sentFilename[0];

            if ($sentFilename !== $localFilename) {
              // Filename not matching?! Error!
              $sentFilename = explode('-', $sentFilename);
              $removed = array_shift($sentFilename);
              $sentFilename = implode('-', $sentFilename);
            }

            if ($sentFilename !== $localFilename) {
              // Still not a match
            } else {
              $downloadImage = download_url($imageUrl);

              if (is_wp_error($downloadImage)) {
                $this->writeLog('Unable to download Image');
                $this->writeLog($imageUrl);
                $this->writeLog($downloadImage);

                $alreadyRestored[] = $imageUrl;
                continue;
              }

              if (file_exists($imagePath)) {
                unlink($imagePath);
              }

              copy($downloadImage, $imagePath);
              unset($downloadImage);

	            // Delete webP if exists
	            $imagesCompressed = get_post_meta($imageID, 'wpc_images_compressed', true);
	            foreach ($imagesCompressed as $image => $data){
		            if ( file_exists( $data['webp_path'] ) ) {
			            unlink( $data['webp_path'] );
		            }
	            }

              $this->writeLog('WebP path ' . $data['webp_path']);
              $this->writeLog('WebP path exists ' . file_exists($data['webp_path']));

            }
          }

          $originalFilePath = wp_get_original_image_path($imageID);
          $oldMeta = wp_generate_attachment_metadata($imageID, $originalFilePath);

          wp_update_attachment_metadata($imageID, $oldMeta);

          // Remove meta tags
          delete_post_meta($imageID, 'wpc_images_compressed');
          delete_post_meta($imageID, 'ic_stats');
          delete_post_meta($imageID, 'ic_compressed_images');
          delete_post_meta($imageID, 'ic_compressed_thumbs');
          delete_post_meta($imageID, 'ic_backup_images');
          update_post_meta($imageID, 'ic_status', 'restored');
          delete_post_meta($imageID, 'ic_bulk_running');
          delete_transient('wps_ic_compress_' . $imageID);

          // Add for heartbeat to pickup
          set_transient('wps_ic_heartbeat_' . $imageID, array('imageID' => $imageID, 'status' => 'restored'), 60);

          $this->writeLog('Ended Image ID - restored ' . $imageID);

          if ($output == 'json') {
            wp_send_json_success(array('msg' => 'backup-restored'));
          }
        }
      } else {
        $this->writeLog('Ended Image ID - failed to get backup ' . $imageID);
        if ($output == 'json') {
          wp_send_json_error(array('msg' => 'failed-to-get-backup', 'apiUrl' => self::$apiURL, 'apikey' => self::$apiParams['apikey'], 'imageID' => $imageID));
        }
      }

    } else {
      $this->writeLog('API Response not 200');
      $this->writeLog(print_r(wp_remote_retrieve_body($call), true));
      $this->writeLog('Ended Image ID ' . $imageID);

      // Failure to contact API
      if ($output == 'json') {
        wp_send_json_error(array('msg' => 'unable-to-contact-api'));
      }
    }
  }

  public function olderBackup($imageID)
  {
    $backup_images = get_post_meta($imageID, 'ic_backup_images', true);

    if (!empty($backup_images) && is_array($backup_images)) {
      $compressed_images = get_post_meta($imageID, 'ic_compressed_images', true);

      // Remove Generated Images
      if (!empty($compressed_images)) {

        foreach ($compressed_images as $index => $path) {
          if (strpos($index, 'webp') !== false) {
            if (file_exists($path)) {
              unlink($path);
            }
          }
        }

      }

      $upload_dir = wp_get_upload_dir();
      $sizes = get_intermediate_image_sizes();
      foreach ($sizes as $i => $size) {
        clearstatcache();
        $image = image_get_intermediate_size($imageID, $size);
        if ($image['path']) {
          $path = $upload_dir['basedir'] . '/' . $image['path'];
          if (file_exists($path)) {
            unlink($path);
          }
        }
      }

      $path_to_image = get_attached_file($imageID);

      // Restore only full
      $restore_image_path = $backup_images['full'];

      // If backup file exists
      if (file_exists($restore_image_path)) {
        unlink($path_to_image);

        // Restore from local backups
        $copy = copy($restore_image_path, $path_to_image);

        // Delete the backup
        unlink($restore_image_path);
      }

      clearstatcache();

      wp_update_attachment_metadata($imageID, wp_generate_attachment_metadata($imageID, $path_to_image));

      // Remove meta tags
      delete_post_meta($imageID, 'ic_stats');
      delete_post_meta($imageID, 'ic_compressed_images');
      delete_post_meta($imageID, 'ic_compressed_thumbs');
      delete_post_meta($imageID, 'ic_backup_images');
      update_post_meta($imageID, 'ic_status', 'restored');

      return true;
    }

    return false;
  }

  public function disable_scaling()
  {
    return false;
  }

	public function singleCompressV4($imageID, $output = 'json')
	{
		wp_raise_memory_limit('image');

		// Is the image type supported
		if (!$this->is_supported($imageID)) {
			if ($output == 'json') {
				wp_send_json_error(array('msg' => 'file-not-supported'));
			} else {
				return 'file-not-supported';
			}
		}

		// Is the image already Compressed
		if ($this->is_already_compressed($imageID)) {
			$media_library = new wps_ic_media_library_live();
			$html = $media_library->compress_details($imageID);

			if ($output == 'json') {
				wp_send_json_error(array('msg' => 'file-already-compressed', 'imageID' => $imageID, 'html' => $html));
			} else {
				return 'file-already-compressed';
			}
		}

		// Set the image status
		set_transient('wps_ic_compress_' . $imageID, ['imageID' => $imageID, 'status' => 'compressing'], 60);

		// Save OLD post meta for restore usage
		if (!get_post_meta($imageID, 'wpc_old_meta')) {
			$oldMeta = wp_get_attachment_metadata($imageID);
			update_post_meta($imageID, 'wpc_old_meta', $oldMeta);
		}

		// Prepare the request params
		$post_fields = array('action' => 'doSingleImage', 'apikey' => self::$apiParams['apikey'], 'imageID' => $imageID,
		                     'siteurl' => self::$siteUrl, 'parameters' => json_encode(['maxWidth' => WPS_IC_MAXWIDTH, 'quality' => self::$apiParams['quality'], 'retina' => self::$apiParams['retina'], 'webp' => self::$apiParams['webp']]));

		// Notify API to queue to queue the request
		$notify = wp_remote_post(self::$apiURL, ['timeout' => 90, 'blocking' => true, 'body' => $post_fields, 'sslverify' =>
			false, 'user-agent' => WPS_IC_API_USERAGENT]);

		if (wp_remote_retrieve_response_code($notify) == 200) {
			// All good, let's wait for queue
			if ($output == 'json') {
				wp_send_json_success( array('waiting-queue','call' => print_r( $notify, true )) );
			}
		} else {
			delete_transient('wps_ic_compress_' . $imageID);
			// We were unable to contact API
			if ($output == 'json') {
				wp_send_json_error( array( 'msg' => 'unable-to-contact-api' , 'call' => print_r( $notify, true )) );
			}
		}
	}

  public function singleCompressV3($imageID, $output = 'json')
  {
    wp_raise_memory_limit('image');
    $settings = get_option(WPS_IC_SETTINGS);

    // Is the image type supported
    if (!$this->is_supported($imageID)) {
      if ($output == 'json') {
        wp_send_json_error(array('msg' => 'file-not-supported'));
      } else {
        return 'file-not-supported';
      }
    }

    // Is the image already Compressed
    if ($this->is_already_compressed($imageID)) {
      $media_library = new wps_ic_media_library_live();
      $html = $media_library->compress_details($imageID);

      if ($output == 'json') {
        wp_send_json_error(array('msg' => 'file-already-compressed', 'imageID' => $imageID, 'html' => $html));
      } else {
        return 'file-already-compressed';
      }
    }

    // Set the image status
    set_transient('wps_ic_compress_' . $imageID, ['imageID' => $imageID, 'status' => 'compressing'], 60);

    // Save OLD post meta for restore usage
    if (!get_post_meta($imageID, 'wpc_old_meta')) {
      $oldMeta = wp_get_attachment_metadata($imageID);
      update_post_meta($imageID, 'wpc_old_meta', $oldMeta);
    }

	  // Prepare the request params
	  $post_fields = array(
		  'action'     => 'queueSingleImage',
		  'imageID'    => $imageID,
		  'siteUrl'    => self::$siteUrl,
		  'apikey'   => self::$apiParams['apikey'],
		  'parameters' => [
			  'maxWidth' => WPS_IC_MAXWIDTH,
			  'quality'  => self::$apiParams['quality'],
			  'retina'   => self::$apiParams['retina'],
			  'webp'     => self::$apiParams['webp']
		  ],
	  );

    // Notify API to queue to queue the request
    $notify = wp_remote_post(self::$apiURL . 'queueManager.php', array('timeout' => 60, 'method' => 'POST', 'sslverify' => false, 'body' => $post_fields, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));

    if (wp_remote_retrieve_response_code($notify) == 200) {
      // All good, let's wait for queue
      wp_send_json_success('waiting-queue');
    } else {
      delete_transient('wps_ic_compress_' . $imageID);
      // We were unable to contact API
      wp_send_json_error(array('msg' => 'unable-to-contact-api'));
    }
  }

  public function compress_image($imageID, $bulk = true, $retina = true, $webp = true, $just_thumbs = false, $regenerate = true, $output = 'json')
  {
    global $wpc_filesystem;
    wp_raise_memory_limit('image');

    $bulkStats = get_transient('wps_ic_bulk_stats');

    // Is the image type supported
    if (!$this->is_supported($imageID)) {
      if (!$bulk) {
        if ($output == 'json') {
          wp_send_json_error(array('msg' => 'file-not-supported'));
        } else {
          return 'file-not-supported';
        }
      }

      return $bulkStats;
    }

    // Is the image already Compressed
    if ($this->is_already_compressed($imageID)) {
      if (!$bulk) {
        $media_library = new wps_ic_media_library_live();
        $html = $media_library->compress_details($imageID);

        if ($output == 'json') {
          wp_send_json_error(array('msg' => 'file-already-compressed', 'imageID' => $imageID, 'html' => $html));
        } else {
          return 'file-already-compressed';
        }
      }

      return $bulkStats;
    }

    // Is the image in process
    $inProcess = get_post_meta($imageID, 'ic_bulk_running', true);
    if ($inProcess && $inProcess == 'true') {
      if ($output == 'json') {
        wp_send_json_error(array('msg' => 'file-in-bulk', 'imageID' => $imageID));
      } else {
        return 'file-in-bulk';
      }
    }

    set_transient('wps_ic_compress_' . $imageID, ['imageID' => $imageID, 'status' => 'compressing'], 30);

    if (!get_post_meta($imageID, 'wpc_old_meta')) {
      $oldMeta = wp_get_attachment_metadata($imageID);
      update_post_meta($imageID, 'wpc_old_meta', $oldMeta);
    }

    $stats = get_post_meta($imageID, 'ic_stats', true);
    if (empty($stats) || !$stats) {
      $stats = array();
    }

    $post_fields = array('action' => 'compressArray', 'imageID' => $imageID, 'siteUrl' => self::$siteUrl, 'maxWidth' => WPS_IC_MAXWIDTH, 'apikey' => self::$apiParams['apikey'], 'quality' => self::$apiParams['quality'], 'retina' => self::$apiParams['retina'], 'webp' => self::$apiParams['webp'],);

    $response = wp_remote_post(self::$apiURL, array('timeout' => 60, 'method' => 'POST', 'sslverify' => false, 'body' => $post_fields, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));

    if (wp_remote_retrieve_response_code($response) == 200) {
      set_transient('wps_ic_compress_' . $imageID, 'sent-to-api', 30);

      $body = wp_remote_retrieve_body($response);
      $body = json_decode($body);

      if ($body->success == 'true') {
        // All good
        if ($output == 'json') {
          wp_send_json_success(array(self::$apiURL, $post_fields, $body));
        } else {
          return 'done';
        }
      } else {
        delete_transient('wps_ic_compress_' . $imageID);

        // Error?
        if ($output == 'json') {
          wp_send_json_error(array('msg' => $body->data->msg, 'server' => $body->data->server));
        } else {
          return 'done';
        }
      }

    } else {
      delete_transient('wps_ic_compress_' . $imageID);

      // We were unable to contact API
      wp_send_json_error(array('msg' => 'unable-to-contact-api'));
    }
  }

  public function debug_msg($attachmentID, $mesage)
  {
    if (defined('WPS_IC_DEBUG') && WPS_IC_DEBUG == 'true') {
      $debug_log = get_post_meta($attachmentID, 'ic_debug', true);
      if (!$debug_log) {
        $debug_log = array();
      }
      $debug_log[] = $mesage;
      update_post_meta($attachmentID, 'ic_debug', $debug_log);
    }
  }

  public function generate_retina($arg)
  {
    $imageID = $arg;
    $return = array();
    $compressed = array();
    $filename = '';

    $image = $image_url = wp_get_attachment_image_src($imageID, 'full');
    $image_url = $image_url[0];

    if ($filename == '') {
      if (strpos($image_url, '.jpg') !== false) {
        $extension = 'jpg';
      } elseif (strpos($image_url, '.jpeg') !== false) {
        $extension = 'jpeg';
      } elseif (strpos($image_url, '.gif') !== false) {
        $extension = 'gif';
      } elseif (strpos($image_url, '.png') !== false) {
        $extension = 'png';
      } else {
        return true;
      }
    }

    /**
     * Figure out the actual file path
     */
    $file_path = get_attached_file($imageID);
    $file_basename = basename($image[0]);
    $file_path = str_replace($file_basename, '', $file_path);

    foreach ($this->sizes as $i => $size) {
      if (empty($image_url)) {
        continue;
      }

      $retinaAPIUrl = self::$apiURL . $image_url;

      if ($size == 'full') {
        continue;
      } else {
        $image = image_get_intermediate_size($imageID, $size);
        $image_url = $image['url'];
      }

      if (empty($image['width']) || $image['width'] == '') {
        continue;
      }

      $file_location = $file_path . basename($image_url);

      // Retina File Path
      $retina_file_location = str_replace('.' . $extension, '-x2.' . $extension, $file_location);

      // Enable Retina
      $retinaAPIUrl = str_replace('r:0', 'r:1', $retinaAPIUrl);
      $retinaAPIUrl = str_replace('w:1', 'w:' . $image['width'], $retinaAPIUrl);

      $call = wp_remote_get($retinaAPIUrl, array('timeout' => 60, 'sslverify' => false, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));

      if (wp_remote_retrieve_response_code($call) == 200) {
        $body = wp_remote_retrieve_body($call);
        if (!empty($body)) {
          file_put_contents($retina_file_location, $body);
          clearstatcache();

          $stats[$size . '-2x']['compressed']['size'] = filesize($retina_file_location);
          $compressed[$size . '-2x'] = $retina_file_location;
        }
      }
    }

    if (isset ($stats)) {
      $return['stats'] = $stats;
    }
    $return['compressed'] = $compressed;

    $stats = get_post_meta($imageID, 'ic_stats', true);

    if (empty($stats)) {
      $stats = array();
    }
    if (empty($return['stats'])) {
      $return['stats'] = array();
    }

    $stats = array_merge($stats, $return['stats']);
    update_post_meta($imageID, 'ic_stats', $stats);

    $compressed = get_post_meta($imageID, 'ic_compressed_images', true);
    $compressed = array_merge($compressed, $return['compressed']);
    update_post_meta($imageID, 'ic_compressed_images', $compressed);

    return $return;
  }

  public function regenerate_thumbnails($imageID)
  {
    wp_raise_memory_limit('image');
    $thumbs = array();
    $thumbs['total']['old'] = 0;
    $thumbs['total']['new'] = 0;

    if (!function_exists('wp_generate_attachment_metadata')) {
      require_once ABSPATH . 'wp-admin/includes/image.php';
    }

    if (!function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    // Get all thumb sizes
    $upload_dir = wp_get_upload_dir();
    $sizes = get_intermediate_image_sizes();
    foreach ($sizes as $i => $size) {
      clearstatcache();
      $image = image_get_intermediate_size($imageID, $size);
      if (!empty($image) && isset($image['path'])) {
        $image['path'] = str_replace('./', '', $image['path']);
        $path = $upload_dir['basedir'] . '/' . $image['path'];
        $thumbs[$size]['old'] = filesize($path);
        $thumbs['total']['old'] = $thumbs['total']['old'] + filesize($path);
      } else if (!empty($image)) {
        $image = str_replace('./', '', $image);
        $path = $upload_dir['basedir'] . '/' . $image;
        $thumbs[$size]['old'] = filesize($path);
        $thumbs['total']['old'] = $thumbs['total']['old'] + filesize($path);
      }
    }

    add_filter('jpeg_quality', function ($arg) {
      return 70;
    });

    foreach ($sizes as $i => $size) {
      clearstatcache();
      $image = image_get_intermediate_size($imageID, $size);
      if (!empty($image) && isset($image['path'])) {
        $image['path'] = str_replace('./', '', $image['path']);
        $path = $upload_dir['basedir'] . '/' . $image['path'];
        $thumbs[$size]['new'] = filesize($path);
        $thumbs['total']['new'] = $thumbs['total']['new'] + filesize($path);
      } else if (!empty($image)) {
        $image = str_replace('./', '', $image);
        $path = $upload_dir['basedir'] . '/' . $image;
        $thumbs[$size]['new'] = filesize($path);
        $thumbs['total']['new'] = $thumbs['total']['new'] + filesize($path);
      }

    }

    update_post_meta($imageID, 'ic_compressed_thumbs', $thumbs);
  }

	public function restartCompressWorker(){
		// Prepare the request params
		$post_fields = array(
			'action'     => 'restartCompressWorker',
			'apikey'     => self::$apiParams['apikey'],
			'siteurl'    => self::$siteUrl,
		);

		// Notify API to queue to queue the request
		$notify = wp_remote_post( self::$apiURL, [
			'timeout'    => 90,
			'blocking'   => true,
			'body'       => $post_fields,
			'sslverify'  =>
				false,
			'user-agent' => WPS_IC_API_USERAGENT
		] );
	}

	public function restartRestoreWorker(){
		// Prepare the request params
		$post_fields = array(
			'action'     => 'restartRestoreWorker',
			'apikey'     => self::$apiParams['apikey'],
			'siteurl'    => self::$siteUrl,
		);

		// Notify API to queue to queue the request
		$notify = wp_remote_post( self::$apiURL, [
			'timeout'    => 90,
			'blocking'   => true,
			'body'       => $post_fields,
			'sslverify'  =>
				false,
			'user-agent' => WPS_IC_API_USERAGENT
		] );
	}

}